#include <SymbianUnit.h>
#include <utf.h>

enum TSymbianUnitPanic
	{
	ESymbianUnitNullPointer
	};

GLDEF_C void Panic(TSymbianUnitPanic aPanic)
	{
	User::Panic(_L("SymbianUnit"), aPanic);
	}

//
//Test
//

EXPORT_C void CTest::SetName(const TDesC& aName)
	{
	iName.Copy(aName);
	}

EXPORT_C const TDesC& CTest::Name() const
	{
	return iName;
	}

//
//TestFailure
//

EXPORT_C CTestFailure* CTestFailure::NewL(const TDesC& aMessage, const TDesC& aFileName, const TUint aLineNumber)
	{
	CTestFailure* self = new (ELeave)CTestFailure();
	CleanupStack::PushL(self);
	self->ConstructL(aMessage, aFileName, aLineNumber);
	CleanupStack::Pop(self);
	return self;
	}

CTestFailure::CTestFailure()
	{
	}

void CTestFailure::ConstructL(const TDesC& aMessage, const TDesC& aFileName, const TUint aLineNumber)
	{
	iMessage = aMessage.AllocL();
	iFileName = aFileName.AllocL();
	iLineNumber = aLineNumber;
	}

EXPORT_C CTestFailure::~CTestFailure()
	{
	delete iMessage;
	delete iFileName;
	}

EXPORT_C const TDesC& CTestFailure::Message() const
	{
	return *iMessage;
	}

EXPORT_C const TDesC& CTestFailure::FileName() const
	{
	return *iFileName;
	}

EXPORT_C TUint CTestFailure::LineNumber() const
	{
	return iLineNumber;
	}

//
//TestResult
//

EXPORT_C CTestResult* CTestResult::NewLC()
	{
	CTestResult* self = new (ELeave)CTestResult();
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

EXPORT_C CTestResult* CTestResult::NewL()
	{
	CTestResult* self = NewLC();
	CleanupStack::Pop(self);
	return self;
	}

CTestResult::CTestResult()
	: iRun(0), iListener(NULL)
	{
	}

void CTestResult::ConstructL()
	{
	iFailures = new (ELeave)CArrayPtrFlat<const CTestFailure>(10);
	}

EXPORT_C CTestResult::~CTestResult()
	{
	iFailures->ResetAndDestroy();
	delete iFailures;
	}

EXPORT_C void CTestResult::AddListenerL(MTestListener& aListener)
	{
	iListener = &aListener;
	}

EXPORT_C void CTestResult::StartTest(const CTest& aTest)
	{
	iRun++;
	if (iListener != NULL)
		{
		iListener->StartTest(aTest);
		}
	}

EXPORT_C void CTestResult::EndTest(const CTest& aTest)
	{
	if (iListener != NULL)
		{
		iListener->EndTest(aTest);
		}
	}

EXPORT_C void CTestResult::StartSuite(const CTest& aTest)
	{
	if (iListener != NULL)
		{
		iListener->StartSuite(aTest);
		}
	}

EXPORT_C void CTestResult::EndSuite(const CTest& aTest)
	{
	if (iListener != NULL)
		{
		iListener->EndSuite(aTest);
		}
	}

EXPORT_C void CTestResult::AddFailureL(const CTestFailure* aFailure)
	{
	iFailures->AppendL(aFailure);
	}

EXPORT_C TBool CTestResult::HasFailure() const
	{
	return FailCount() > 0;
	}

EXPORT_C TUint CTestResult::RunCount() const
	{
	return iRun;
	}

EXPORT_C TUint CTestResult::FailCount() const
	{
	return iFailures->Count();
	}

EXPORT_C TUint CTestResult::PassCount() const
	{
	return RunCount() - FailCount();
	}

EXPORT_C const CTestFailure& CTestResult::Failure(TUint index) const
	{
	return *(iFailures->At(index));
	}

//
//TestCase
//

EXPORT_C void CTestCase::SetUpL()
	{
	}

EXPORT_C void CTestCase::TearDown()
	{
	}

EXPORT_C void CTestCase::RunL(CTestResult& aResult)
	{
	CTestSuite* suite = SuiteLC();
	suite->RunL(aResult);
	CleanupStack::PopAndDestroy(suite);
	}

EXPORT_C void CTestCase::_Failed(const char* aMessage, const char* aFileName, const TUint aLineNumber)
	{
	TBuf16<128> messageBuf;
	CnvUtfConverter::ConvertToUnicodeFromUtf8(messageBuf, TPtrC8((const TText8*)aMessage));

	TBuf16<128> fileNameBuf;
	CnvUtfConverter::ConvertToUnicodeFromUtf8(fileNameBuf, TPtrC8((const TText8*)aFileName));

	__iResult->AddFailureL(CTestFailure::NewL(messageBuf, fileNameBuf, aLineNumber));

	User::Leave(KErrAssertionFailed);\
	}


//
//TestSuite
//

EXPORT_C CTestSuite* CTestSuite::NewLC()
	{
	CTestSuite* self = new (ELeave)CTestSuite();
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

EXPORT_C CTestSuite* CTestSuite::NewL()
	{
	CTestSuite* self = NewLC();
	CleanupStack::Pop(self);
	return self;
	}

CTestSuite::CTestSuite()
	{
	}

void CTestSuite::ConstructL()
	{
	iTests = new (ELeave)CArrayPtrFlat<CTest>(10);
	}

EXPORT_C CTestSuite::~CTestSuite()
	{
	iTests->ResetAndDestroy();
	delete iTests;
	}

EXPORT_C void CTestSuite::AddTestL(CTest* aTest)
	{
	iTests->AppendL(aTest);
	}

EXPORT_C void CTestSuite::RunL(CTestResult& aResult)
	{
	aResult.StartSuite(*this);
	for (TInt i = 0; i < iTests->Count(); i++)
		{
		iTests->At(i)->RunL(aResult);
		}
	aResult.EndSuite(*this);
	}
