#include "SymbianUnitTest.h"

class WasRun : public CTestCase
	{
public:
	TBool iWasRun;

	static WasRun* NewL()
		{
		WasRun* self = new (ELeave)WasRun();
		return self;
		}

	SUNIT_TESTCASE_TABLE_BEGIN(WasRun)
		SUNIT_TESTCASE(TestMethod)
	SUNIT_TESTCASE_TABLE_END

	virtual void TestMethod()
		{
		iWasRun = ETrue;
		}
	};

class WasFail : public CTestCase
	{
public:
	TBool iWasRun;

	static WasFail* NewL()
		{
		WasFail* self = new (ELeave)WasFail();
		return self;
		}

	SUNIT_TESTCASE_TABLE_BEGIN(WasFail)
		SUNIT_TESTCASE(TestMethod)
	SUNIT_TESTCASE_TABLE_END

	void TestMethod()
		{
		SUNIT_ASSERT(3 == 4);
		iWasRun = ETrue;
		}
	};

class SetUpAndTearDown : public CTestCase
	{
public:
	TBool iWillFail;
	TBool iWasSetUp;
	TBool iWasRun;
	TBool iWasTearDown;

	static SetUpAndTearDown* NewL()
		{
		SetUpAndTearDown* self = new (ELeave)SetUpAndTearDown();
		return self;
		}

	SUNIT_TESTCASE_TABLE_BEGIN(SetUpAndTearDown)
		SUNIT_TESTCASE(TestMethod)
	SUNIT_TESTCASE_TABLE_END

	void SetUpL()
		{
		iWasSetUp = ETrue;
		}

	void TearDown()
		{
		iWasTearDown = ETrue;
		}

	void TestMethod()
		{
		iWasRun = ETrue;
		if (iWillFail) SUNIT_ASSERT(EFalse);
		}
	};

class ChildTest : public CTestCase
	{
public:
	SUNIT_TESTCASE_TABLE_DECLARE;
	void TestSomething();
	};

SUNIT_TESTCASE_TABLE_BEGIN(ChildTest)
	SUNIT_TESTCASE(TestSomething)
SUNIT_TESTCASE_TABLE_END

void ChildTest::TestSomething()
	{
	}

SUNIT_TESTCASE_TABLE_BEGIN(SymbianUnitTest)
	SUNIT_TESTCASE(TestWasRun)
	SUNIT_TESTCASE(TestWasFail)
	SUNIT_TESTCASE(TestTestResultResize)
	SUNIT_TESTCASE(TestTestSuite)
	SUNIT_TESTCASE(TestTestSuiteResize)
	SUNIT_TESTCASE(TestSetUpAndTearDown)
	SUNIT_TESTCASE(TestSetUpTearDownAndFailingRun)
	SUNIT_TESTCASE(TestTestListener)
	CTest* child = new (ELeave) ChildTest();
	CleanupStack::PushL(child);
	suite->AddTestL(child);
	CleanupStack::Pop(child);
SUNIT_TESTCASE_TABLE_END

void SymbianUnitTest::TestWasRun()
	{
	CTestResult* result = CTestResult::NewLC();
	WasRun test;
	test.iWasRun = EFalse;
	test.RunL(*result);
	SUNIT_ASSERT(test.iWasRun);
	CleanupStack::PopAndDestroy(result);
	}

void SymbianUnitTest::TestWasFail()
	{
	CTestResult* result = CTestResult::NewLC();
	WasFail test;
	test.iWasRun = EFalse;
	test.RunL(*result);
	SUNIT_ASSERT(!test.iWasRun);
	SUNIT_ASSERT(1 == result->FailCount());
	SUNIT_ASSERT(_L("3 == 4") == result->Failure(0).Message());
	SUNIT_ASSERT(_L("SymbianUnitTest.cpp").CompareF(result->Failure(0).FileName()) == 0);
	CleanupStack::PopAndDestroy(result);
	}

void SymbianUnitTest::TestTestResultResize()
	{
	CTestResult* result = CTestResult::NewLC();
	TInt capacity = 10; //result->iCapacity;
	for (TInt i = 0; i < capacity + 1; i++)
		{
		result->AddFailureL(CTestFailure::NewL(_L(""), _L(""), 0));
		}
	//SUNIT_ASSERT(capacity * 2 == result->iCapacity);
	SUNIT_ASSERT(capacity + 1 == result->FailCount());
	CleanupStack::PopAndDestroy(result);
	}

void SymbianUnitTest::TestTestSuite()
	{
	CTestResult* result = CTestResult::NewLC();
	CTestSuite* suite = CTestSuite::NewLC();
	suite->AddTestL(WasRun::NewL());
	suite->AddTestL(WasFail::NewL());
	suite->RunL(*result);
	SUNIT_ASSERT(2 == result->RunCount());
	SUNIT_ASSERT(1 == result->FailCount());
	CleanupStack::PopAndDestroy(suite);
	CleanupStack::PopAndDestroy(result);
	}

void SymbianUnitTest::TestTestSuiteResize()
	{
	CTestResult* result = CTestResult::NewLC();
	CTestSuite* suite = CTestSuite::NewLC();
	TInt capacity = 10; //suite->iCapacity;
	for (TInt i = 0; i < capacity + 1; i++)
		{
		suite->AddTestL(WasRun::NewL());
		}
	//SUNIT_ASSERT(capacity * 2 == suite->iCapacity);
	//SUNIT_ASSERT(capacity + 1 == suite->iSize);
	CleanupStack::PopAndDestroy(suite);
	CleanupStack::PopAndDestroy(result);
	}

void SymbianUnitTest::TestSetUpAndTearDown()
	{
	CTestResult* result = CTestResult::NewLC();
	SetUpAndTearDown test;
	test.iWillFail = EFalse;
	test.iWasSetUp = EFalse;
	test.iWasRun = EFalse;
	test.iWasTearDown = EFalse;
	test.RunL(*result);
	SUNIT_ASSERT(test.iWasSetUp);
	SUNIT_ASSERT(test.iWasRun);
	SUNIT_ASSERT(test.iWasTearDown);
	CleanupStack::PopAndDestroy(result);
	}

void SymbianUnitTest::TestSetUpTearDownAndFailingRun()
	{
	CTestResult* result = CTestResult::NewLC();
	SetUpAndTearDown test;
	test.iWillFail = ETrue;
	test.iWasSetUp = EFalse;
	test.iWasRun = EFalse;
	test.iWasTearDown = EFalse;
	test.RunL(*result);
	SUNIT_ASSERT(test.iWasSetUp);
	SUNIT_ASSERT(test.iWasRun);
	SUNIT_ASSERT(test.iWasTearDown);
	CleanupStack::PopAndDestroy(result);
	}

class TDummyTestListener : public CBase, public MTestListener 
	{
public:
	TBuf<256> iLog;

	void StartTest(const CTest& aTest)
		{
		iLog.AppendFormat(_L("%S Start\n"), &aTest.Name());
		}

	void EndTest(const CTest& aTest)
		{
		iLog.AppendFormat(_L("%S End\n"), &aTest.Name());
		}

	void StartSuite(const CTest& aTest)
		{
		iLog.AppendFormat(_L("%S Start\n"), &aTest.Name());
		}

	void EndSuite(const CTest& aTest)
		{
		iLog.AppendFormat(_L("%S End\n"), &aTest.Name());
		}
	};

void SymbianUnitTest::TestTestListener()
	{
	CTestResult* result = CTestResult::NewLC();
	TDummyTestListener listener;
	result->AddListenerL(listener);
	WasRun test;
	test.RunL(*result);
	SUNIT_ASSERT(listener.iLog == TPtrC(L"WasRun Start\nTestMethod Start\nTestMethod End\nWasRun End\n"));
	CleanupStack::PopAndDestroy(result);
	}

EXPORT_C CTest* NewTest()
	{
	return new SymbianUnitTest();
	}

#ifndef EKA2
GLDEF_C TInt E32Dll(TDllReason /*aReason*/)
{
return KErrNone;
}
#endif //EKA2
